#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2008 Manuel Holtgrewe, Mathias Ziebarth, Lucas Lürich, Joachim Breitner
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


"""The statistics modules' code allows to record certain values and generate
statistics about it.

"""

class Statistics(object):
    """
    Statistics objects allow to record values and later on print statistics
    about them.
    
    """
    
    def __init__(self):
        """Initialization."""
        self.once_rule_applications = 0
        self.ordinary_rule_applications = 0
        self.recursions = 0
        self.recursion_returns = 0
    
    def registerOrdinaryRuleApplication(self, rule):
        """Register the application of an ordinary rule."""
        self.ordinary_rule_applications += 1
    
    def registerOnceRuleApplication(self, rule):
        """Register the application of an 'apply-once' rule."""
        self.once_rule_applications += 1
    
    def registerRecursion(self):
        """Register a recursive solve call."""
        self.recursions += 1
    
    def registerReturnFromRecursion(self):
        """Register a return of a solve call."""
        self.recursion_returns += 1
    
    def toString(self):
        """Prints simple statistics."""
        result = []
        result.append("Statistics")
        result.append("----------")
        result.append("")
        result.append("once rule applications:     %d" % \
            self.once_rule_applications)
        result.append("ordinary rule applications: %d" % \
            self.ordinary_rule_applications)
        result.append("recursion:                  %d" % self.recursions)
        result.append("recursion returns:          %d" % \
            self.recursion_returns)
        
        return "\n".join(result)
    
