{-# LANGUAGE FlexibleInstances #-}

{- (c) 2008 Joachim Breitner
 -
 - This haskell Program generates input in the informaticup-syntax
 - for Kreuzzahlenraetsel that simulate a sudoku
 -}

import Control.Monad

unary name v = name++"("++v++")"
binary name a b = name++"("++a++","++b++")"

-- Use nice syntax instead of ugly syntax
instance Num [Char] where
	(+) = binary "plus"
	(-) = binary "minus"
	(*) = binary "mal"
	abs = undefined
	signum = undefined
	fromInteger = show

-- powers of 10
l `e` p = fromInteger (l * 10^p)

-- Carry-Over safe Version of v
safe v 8 = v
safe v k = v -  1 `e` (k+1)

-- More operators
qs = unary "qs"
qp = unary "qp"
u  = unary "U"

-- equality
eq  = binary "="

-- not negative
nn v = eq (qs (v*10+1)) (qs v + 1)

-- no zero
nz v = nn(qp v - 1)

s v k l = qs(safe v k + l `e` k) - qs(safe v k)

delta' a b k1 k2 l = s a k1 l - s b k2 l

delta a b k1 k2  = sum $ map (delta' a b k1 k2) [1..9]

digits_differ (a, b, k1, k2) = nn (qs (delta a b k1 k2) - 1)

rows = map (:"_w") ['A'..'I']

column_pairs = do
	row1 <- rows
	row2 <- rows
	guard  $ row1 < row2
	k <- [0..8]
	return (row1, row2, k, k)

block_pairs = do
	x1 <- [0..8]
	x2 <- [0..8]
	y1 <- [0..8]
	y2 <- [0..8]
	guard $ y1 < y2 -- Rows handled by U(..)
	guard $ x1 /= x2 -- Columns already handled as well
	guard $ x1 `div` 3 == x2 `div` 3 && -- In same block?
	        y1 `div` 3 == y2 `div` 3 

	let row1 = rows !! y1
	    row2 = rows !! y2
	return (row1, row2, x1, x2)

all_pairs = column_pairs ++ block_pairs

pair_rules = map digits_differ all_pairs

uniqe_rules = map u rows

nz_rules = map nz rows

all_rules = uniqe_rules ++ nz_rules ++ pair_rules

cells = zipWith (\c n -> c:" "++show n ++" 1") ['A'..'I'] [1..9]

descr = ["9 9", "9"] ++ cells ++
	["9"] ++ map (:[]) ['A'..'I'] ++
	["0", show (length all_rules) ] ++
	all_rules

main = mapM_ putStrLn descr
